
__version__ = "$Id: constants.py 45 2018-04-21 17:56:18Z lbusoni $"


class Constants:
    METER_2_NANOMETER= 1e9
    APP_NAME= "inaf.arcetri.ao.palpao"
    APP_AUTHOR= "INAF Arcetri Adaptive Optics"
    THIS_PACKAGE= 'palpao'

